<?php

namespace Drupal\ckeditor_mentions;

use Drupal\ckeditor5\Plugin\CKEditor5PluginManager;
use Drupal\ckeditor_mentions\Events\CKEditorMentionsEvent;
use Drupal\ckeditor_mentions\MentionsType\MentionsTypeManagerInterface;
use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Entity\EntityInterface;
use Drupal\Core\Entity\EntityRepositoryInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Entity\FieldableEntityInterface;
use Drupal\Core\Session\AccountInterface;
use Masterminds\HTML5;
use Symfony\Component\EventDispatcher\EventDispatcherInterface;

/**
 * Class MentionService.
 *
 * @package Drupal\ckeditor_mentions
 * @internal
 */
class MentionEventDispatcher {

  /**
   * MentionService constructor.
   */
  public function __construct(
      protected AccountInterface $currentUser,
      protected ConfigFactoryInterface $configFactory,
      protected EventDispatcherInterface $eventDispatcher,
      protected EntityTypeManagerInterface $entityManager,
      protected MentionsTypeManagerInterface $mentionsTypeManager,
      protected CKEditor5PluginManager $ckeditorPluginManager,
      protected EntityRepositoryInterface $entityRepository
  ) {
  }

  /**
   * Triggers the Mention Event.
   *
   * @param \Drupal\Core\Entity\EntityInterface $entity
   *   Trigger the mention event.
   * @param string $event_name
   *   The name of the event.
   */
  public function dispatchMentionEvent(EntityInterface $entity, string $event_name): void {
    foreach ($this->getMentionsFromEntity($entity) as $mentioned_entity) {
      $event = new CKEditorMentionsEvent($entity, $mentioned_entity['entity'], $mentioned_entity['plugin']);
      $this->eventDispatcher->dispatch($event, $event_name);
    }
  }

  /**
   * Reads all the fields from an entity and return all the users mentioned.
   *
   * The array returned has this format:
   *
   * [entity_id] => [
   *    'uuid' => $uuid,
   *   'id' => $id,
   *   'field_name' => [
   *     'delta' => [
   *       0 => 0,
   *       1 => 1,
   *       2 => 2,
   *     ]
   *   ]
   * ];
   *
   * The first key is the user id, the next key is the field_name where the
   * user was mentioned and finally the deltas of the fields.
   *
   * @param \Drupal\Core\Entity\EntityInterface $entity
   *   The entity from which will get the mentions.
   *
   * @return array
   *   The users mentioned.
   */
  public function getMentionsFromEntity(EntityInterface $entity): array {
    $mentioned_entities = [];
    // Check if some of the fields is using the CKEditor editor.
    if (!$entity instanceof FieldableEntityInterface) {
      return $mentioned_entities;
    }

    $bundle_fields = $entity->getFieldDefinitions();
    $format_using_mentions = $this->getTexformatsUsingMentions();

    foreach ($bundle_fields as $field_name => $field) {
      $field_value = $entity->get($field_name)->getValue();
      foreach ($field_value as $key => $item) {
        if (isset($item['format']) && in_array($item['format'], $format_using_mentions)) {
          foreach ($this->getMentionedEntities($item['value']) as $mentioned_entity_information) {
            $mentioned_item = [
              'field_info' => [
                $field_name => [
                  'delta' => [
                    $key => $key,
                  ],
                ],
              ],
            ];
            $mentioned_item += $mentioned_entity_information;
            $mentioned_entities[] = $mentioned_item;
          }
        }
      }
    }
    return $mentioned_entities;
  }

  /**
   * Returns the list of text formats using the Mentions plugin.
   *
   * @return array
   *   An array with the editors using the mentions plugin.
   */
  public function getTexformatsUsingMentions(): array {
    $formats_using_mentions = [];
    $editors = $this->entityManager->getStorage('editor')->loadByProperties(['editor' => 'ckeditor5']);

    /** @var \Drupal\editor\Entity\Editor $editor */
    foreach ($editors as $editor) {
      $editorSettings = $editor->getSettings();
      foreach ($editorSettings['plugins']['ckeditor_mentions_mentions']['plugins'] ?? [] as $plugin) {
        if ($plugin['enable'] ?? FALSE) {
          $formats_using_mentions[] = $editor->getFilterFormat()->id();
        }
      }
    }

    return $formats_using_mentions;
  }

  /**
   * Returns an with information about mentioned entities.
   *
   * @param string $field_value
   *   The field text $field_text.
   *
   * @return array
   *   Array with information about mentioned entities.
   */
  public function getMentionedEntities(string $field_value): array {
    $mentioned_entities = [];
    $plugins = [];

    if (empty($field_value)) {
      return $mentioned_entities;
    }

    // Instantiate the HTML5 parser, but without the HTML5 namespace being
    // added to the DOM document.
    $html5 = new HTML5(['disable_html_ns' => TRUE]);
    $dom = $html5->loadHTML($field_value);

    $anchors = $dom->getElementsByTagName('a');
    foreach ($anchors as $anchor) {
      /** @var @todo BC LAYER, with data-entity-id **/
      $plugin = NULL;
      $entity_uuid = $anchor->getAttribute('data-entity-uuid');
      $plugin_id = $anchor->getAttribute('data-plugin');

      if (empty($entity_uuid) || empty($plugin_id)) {
        continue;
      }

      /** @var \Drupal\ckeditor_mentions\MentionsType\MentionsTypeBase $plugin */
      $plugin = $plugins[$plugin_id] ??= $this->mentionsTypeManager->createInstance($plugin_id);

      $mentioned_entities[] = [
        'plugin' => $plugin,
        'entity' => $this->entityRepository->loadEntityByUuid($plugin->getPluginDefinition()['entity_type'], $entity_uuid),
      ];
    }

    return $mentioned_entities;
  }

}
